<?php

namespace Drupal\stop_broken_link_in_body\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Batch\BatchBuilder;

/**
 * Class RunContentBrokenLink.
 */
class RunContentBrokenLink extends FormBase {

  /**
   * Node with broken links.
   */
  public $nodes_broken = [];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'run_content_broken_link';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['content_types'] = [
      '#type' => 'checkboxes',
      '#options' => node_type_get_names(),
      '#title' => $this->t('Content types to check for broken links.'),
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Check broken links'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get content types to check for broken links.
    $content_types = array_filter($form_state->getValue('content_types'), function($value) {
      return $value !== 0;
    });
    // Get node ids.
    $nids = $this->getNodeidstoCheck($content_types);
    // Set batch operations for checking procces.
    batch_set($this->getBrokenLinks($nids));
  }

  /**
   * Get nodes ids.
   *
   * @return array $content_types
   *   Content types to check.
   */
  public function getNodeidstoCheck(array $content_types) {
    if (empty($content_types)) {
      return [];
    }
    $query = \Drupal::entityQuery('node')
      ->condition('status', 1)
      ->condition('type', $content_types, 'IN')
      ->sort('changed', 'DESC');
    $nids = $query->execute();
    return $nids;
  }

  /**
   * Form submission handler for running cron manually.
   */
  public function getBrokenLinks(array $nids) {
    // Delete current broken nodes saved.
    \Drupal::state()->delete('broken_content');
    // Build batch operations, one by node.
    $operations = [];
    // Defining the batch builder.
    $batch_builder = new BatchBuilder();
    $batch_builder->setTitle($this->t('Searching broken links'))
      ->setInitMessage($this->t('Starting...'))
      ->setProgressMessage($this->t('Checking @current contents of @total. Estimated time: @estimate.'))
      ->setErrorMessage($this->t('Error in broken links.'))
      ->setFinishCallback([$this, 'finish']);

    // Loop through the revisions to delete, create batch operations array.
    foreach ($nids as $nid) {
      // Adding the operation.
      $batch_builder->addOperation([$this, 'procces'], [$nid]);
    }
    return $batch_builder->toArray();
  }

  /**
   * Implements callback_batch_operation().
   *
   * Check broken links in the node with $nid.
   *
   * @param $nid
   *   The node id to check.
   * @param array $context
   *   An array of contextual key/values.
   */
  public function procces($nid, &$context) {
    $nodes_broken = $this->getTextValues($nid);
    if (!empty($nodes_broken)) {

      $current_state = \Drupal::state();
      $current_nodes_broken = $current_state->get('broken_content', []);

      $current_nodes_broken[] = $nodes_broken;
      $current_state->set('broken_content', $current_nodes_broken);

    }
  }

  /**
   * Get text values to check in content.
   *
   * @param $nid
   *   The node id to check.
   */
  public function getTextValues($nid) {
    $has_broken_link = [];
    $node = \Drupal\node\Entity\Node::load($nid);
    // Get fields to check.
    $fields_to_check = stop_broken_link_in_body_get_fields_to_checks();
    // Get all node types.
    $node_type = node_type_get_names();
    if ($node) {
      // Check all fields in current node.
      foreach ($fields_to_check as $field_to_check) {
        $field = $node->{$field_to_check};
        if ($field && $field instanceof \Drupal\Core\Field\FieldItemListInterface) {
          // Get current field text value.
          $value = $node->{$field_to_check}->getValue();
          if (isset($value[0]['value'])) {
            $text_to_check = $value[0]['value'];
            // Check for broken links for current fields and text.
            $messages = stop_broken_link_in_body_check_broken_links($field_to_check, $text_to_check);
            // If not empty so there is broken links for current node.
            if (!empty($messages)) {
              $build['url'] = [
                '#type' => 'html_tag',
                '#tag' => 'a',
                '#value' => $node->getTitle(),
                '#attributes' => [
                  'href' => $node->toUrl('edit-form')->toString(),
                  'target' => '_blank',
                ],
              ];
              return [
                'url'  => \Drupal::service('renderer')->render($build),
                'type' => $node_type[$node->getType()],
              ];
            }
          }
        }
      }
    }
    return $has_broken_link;
  }

  /**
   * Finish batch for checking broken links.
   *
   * @param bool $success
   *   Indicate that the batch API tasks were all completed successfully.
   * @param array $results
   *   An array of all the results that were updated in update_do_one().
   * @param array $operations
   *   A list of the operations that had not been completed by the batch API.
   */
  public function finish($success, array $results, array $operations) {
    \Drupal::messenger()->addMessage($this->t('Broken links check completed, for the contents of the selected types.'));
  }

}
